# Perf dashboard API

## Authenticating
The perf dashboard API uses OAuth to authenticate. You must either have a
Google account or a service account to access the API. See
[examples](/dashboard/dashboard/api/examples/) directory
for examples of how to access the API.

## Alerts
URL pattern for accessing alerts:

 * `/api/alerts?<params>`: Get matching alerts. Supported query parameters:
    * `key`: Urlsafe alert entity key.
    * `sheriff`: String. If unspecified, alerts for any sheriff will be
      returned.
    * `bug_id`: Set to `''` in order to select untriaged alerts.
    * `is_improvement`: When omitted, both improvements and regressions are
      returned. When set to `true`, only improvements are returned. When set to
      `false`, only regressions are returned.
    * `recovered`: When omitted, both recovered and unrecovered alerts are
      returned. When set to `true`, only recovered alerts are returned. When set to
      `false`, only unrecovered alerts are returned.
    * `test`: Full slash-separated test path string like
      `master/bot/test_suite/measurement/test_case`.
    * `master`, `bot`, `test_suite`: Optional strings.
    * `limit`: Positive integer. Default 100.
    * `cursor`: Set to the `next_cursor` returned from a previous request in
      order to page through results for queries that match more than `limit`
      alerts.
    * `min_start_revision`, `max_start_revision`: Integers. If unspecified,
      alerts at any revisions will be returned.
    * `min_end_revision`, `max_end_revision`: Integers. If unspecified, alerts
      at any revisions will be returned.
    * `min_timestamp`, `max_timestamp`: Datetimes in ISO format.
    * `report`: Report template id as returned by `/api/report_names`. Multiple
      `report`s may be specified. Filters alerts to the timeseries referenced by
      the reports.

The alerts API returns a JSON list of alerts with the following fields:

 * `date`: Date the alert fired in YYYY-MM-DD format
 * `master`: Buildbot master the alert fired on.
 * `bot`: Buildbot device type the alert fired on (for example, "nexus 7").
 * `test`: Test path of the test the alert was fired on. This contains the
    benchmark/metric/page, like page_cycler/time_to_load/cnn.
 * `ref_test`: If there was a ref build, the test_path of the ref build.
 * `testsuite`: Name of the benchmark the alert was fired on.
 * `recovered`: Whether the dashboard was able to detect that the alert
    recovered. Note that this is not possible to detect perfectly, as perf
    graphs are noisy and there may be multiple performance changes. The
    dashboard errs on the side of false negatives, only marking an alert
    recovered if it finds a change almost exactly opposite of the one in the
    alert.
 * `percent_changed`: The percentage difference before/after the alert.
 * `absolute_delta`: The difference before/after the alert, in `units`.
 * `median_before_anomaly`: The median value of the chart segment before the
    alert, in `units`.
 * `median_after_anomaly`: The median value of the chart segment after the
    alert, in `units`.
 * `units`: The units for the metric that was alerted on (for example,
    milliseconds).
 * `improvement`: true if the alert fired on an improvement; false otherwise.
    (note improvements can be filtered out of the history API with the
    `improvements` param).
 * `start_revision`: Start of the revision range where the change occurred.
 * `end_revision`: End of the revision range where the change occurred.
 * `dashboard_link`: Link to view the chart with the alert on the perf
    dashboard.
 * `bug_id`: The bug linked to the alert in the perf dashboard. -1 means the
    perf sheriff marked the bug invalid, and -2 means ignored (known issue).
    An empty bug id means the alert has not been triaged yet.
 * `group`: Unique id of the group the alert was placed in by the perf
    dashboard.
 * `key`: Unique id of the alert, generated by the perf dashboard.

## File New Bug

Permitted sheriffs can file new bugs for alerts by posting to `/api/new_bug`. It
accepts `owner`, `cc`, `summary`, `description`, zero or more `label`, zero or
more `component`, zero or more alert `key`. It returns json `{bug_id, error}`.

## Assign Alerts to Existing Bug

Permitted sheriffs can assign alerts to an existing bug by posting to
`/api/existing_bug`. It accepts `bug` and one or more `key`.

## Bugs

For internal users only, the bugs API gives data about bugs so that it can
easily be lined up with alerts. URL pattern for accessing bugs:

 * `/api/bugs/id`: Get all the information about bug `id`.
 * `/api/bugs/recent`: Get an array of performance regression bugs created in
   the last 5 days.

The bugs API returns the following JSON about the bug:

 * `author`: Email of the issue author
 * `legacy_bisects`: List of bisects (to be replaced by pinpoint). Each has:
   * `status`: Job status, such as "started" or "failed"
   * `bot`: Name of the bisector the job ran on,
   * `buildbucket_link`: Link to buildbucket page with more info
   * `command`: The command the bisect was run with
   * `metric`: The metric that was bisected
 * `cc`: List of emails cc-ed to the issue
 * `comments`: List of comments. Each has:
   * `content`: Text of the comment
   * `author`: Email of the comment author
   * `published`: Date the comment was published, in milliseconds since 1970
 * `components`: List of components associated with the issue
 * `id`: The same id that was passed in
 * `labels`: List of labels associated with the issue
 * `published`: Date the comment was published, in milliseconds since 1970
 * `state`: Issue state (open or closed)
 * `status`: Status of work on the issue (Assigned, Fixed, etc)
 * `summary`: The issue summary/title text.

## Timeseries

 URL patterns for accessing timeseries:

  * `/api/list_timeseries/benchmark`: List all the monitored timeseries
    associated with `benchmark`. Takes an optional `sheriff` param in postdata,
    which can be set to a name of the seriff rotation or `all` to search for
    metrics in all rotations. If not specified, the `Chromium Perf Sheriff`
    rotation is used. Returns an array of test_paths
    (master/bot/benchmark/metric/page).
  * `/api/timeseries/test_path`: Return the timeseries data for the given
    `test_path` as JSON. Can specify `num_days` param in postdata, defaults to
    30.
  * `/api/test_suites`: Return an array of names of test suites.
  * `/api/describe/test_suite`: Return an object containing `measurements`,
    `bots`, and `cases`, all of which are sorted arrays of strings.
  * `/api/timeseries2?<params>`: Get timeseries data, alerts, Histograms, and
    SparseDiagnostics. Post body parameters may include the following:
     * `test_suite`, `measurement`, `bot`: Required strings from
       `/api/test_suites` and `/api/describe`.
     * `test_case`: Optional string from `/api/describe`.
     * `build_type`: Optional enum `test` (default) or `ref`.
     * `min_revision`, `max_revision`: Optional point id revision numbers.
     * `min_timestamp`, `max_timestamp`: Optional ISO 8601 strings.
     * `columns`: Required comma-separated list of strings. May contain
        * `revision` produces point ID numbers.
        * `avg`, `std`, `count`, `max`, `min`, `sum` produce float numbers.
        * `revisions` produces an object whose keys are names like `r_webkit`
          and `r_chromium`, and whose values are numbers or strings.
        * `timestamp` records when the data was uploaded, and produces ISO 8601
          strings like "2018-07-09T09:58:20.210539"
        * `alert` produces objects as specified in the Alerts section above.
        * `histogram` produces [Histogram
          JSON](https://chromium.googlesource.com/catapult/+/master/docs/histogram-set-json-format.md).
        * `diagnostics` produces an object whose keys are names (which may be
          [reserved
          names](https://chromium.googlesource.com/catapult/+/master/tracing/tracing/value/diagnostics/reserved_infos.py)),
          and whose values are [Diagnostic
          JSON](https://chromium.googlesource.com/catapult/+/master/docs/histogram-set-json-format.md#diagnostics).
    Returns a JSON object containing `units`, `improvement_direction` ("down" or
    "up"), and "data", which is an array. Elements of the data array are arrays
    of values corresponding to the `columns` parameter. For example, if
    `columns=revision,avg,timestamp`, then elements of the data array will be
    3-tuples like `[123456,42.42,"2018-07-09T09:58:20.210539"]`.
  * `/api/report/names`: Return an array of objects `{name, id, modified}`.
  * `/api/report/template`: Creates or modifies report templates.
    Set `template` to a json object `{rows, statistics}`.
    Set `name` to a string.
    Set `owners` to a comma-separated list of email addresses.
    Optionally set `id` to a template id as returned by `/api/report/names` in
    order to modify an existing template.
    Returns the full new list of report templates `{name, id, modified}`.
  * `/api/report/generate`: Generates reports.
    Set `id` to a template id as returned by `/api/report/names`.
    Set `revisions` to a comma-separated list of point id numbers or `latest`.
